# -*- coding: utf-8 -*-
import json
import logging
import requests
from json.decoder import JSONDecodeError

logger = logging.getLogger(__name__)


class ConnectorException(Exception):
    def __init__(self, message, description, code):
        self.code = code
        self.message = message
        self.description = description
        super().__init__(message)


class Connector:

    def __init__(self, headers=None, verify_ssl=True):
        self.headers = headers
        self.verify_ssl = verify_ssl

    def _handle_response(self, response, object_name, url):
        if response.status_code == 400:
            raise ConnectorException(response.content, response.text, 400)
        if response.status_code == 422:
            raise ConnectorException(
                f'Error:  {response.json()}', response.text, 422)
        if response.status_code == 401:
            raise ConnectorException(
                f'Unauthorized operation over {object_name}', response.text, 401)
        if response.status_code == 404:
            raise ConnectorException(
                f'Not found error trying to access to {url}', response.text, 404)
        if response.status_code not in [200, 201]:
            raise ConnectorException(
                f'Fail operation to {url}', response.text, response.status_code,)
        if isinstance(response.json, dict):
            return response.json
        try:
            return json.loads(response.content)
        except JSONDecodeError:
            return response.content.decode('utf-8')

    def get(self, url, object_name='this objects'):
        logger.debug(f'Getting url {url}')
        if self.headers:
            response = requests.get(
                url, headers=self.headers, verify=self.verify_ssl)
        else:
            response = requests.get(url)
        return self._handle_response(response, object_name, url)

    def post(self, url, data, object_name='this objects'):
        logger.debug(f'Creating {object_name} to {url}')
        if self.headers:
            response = requests.post(
                url, json=data, headers=self.headers, verify=self.verify_ssl)
        else:
            response = requests.post(url, json=data, verify=self.verify_ssl)
        return self._handle_response(response, object_name, url)

    def put(self, url, data, object_name):
        logger.debug(f'Updating {object_name} to {url}')
        if self.headers:
            response = requests.put(
                url, json=data, headers=self.headers, verify=self.verify_ssl)
        else:
            response = requests.put(url, json=data, verify=self.verify_ssl)
        return self._handle_response(response, object_name, url)

    def delete(self, url, object_name):
        logger.debug(f'Deleting {object_name} from {url}')
        if self.headers:
            response = requests.delete(
                url, headers=self.headers, verify=self.verify_ssl)
        else:
            response = requests.delete(url, verify=self.verify_ssl)
        return self._handle_response(response, object_name, url)



# -*- coding: utf-8 -*-
import json
import urllib
import logging

from urbano.connector import Connector, ConnectorException
from urbano.settings import api_settings

logger = logging.getLogger(__name__)


class UrbanoHandler:
    """
        Handler to send shipping payload to Urbano
    """

    def __init__(self, base_url=api_settings.URBANO['BASE_URL'],
                 user=api_settings.URBANO['USER'],
                 password=api_settings.URBANO['PASSWORD'],
                 verify=True):

        self.base_url = base_url
        self.user = user
        self.password = password
        self.verify = verify
        self.connector = Connector(self._headers(), verify_ssl=self.verify)

    def _headers(self):
        """
            Here define the headers for all connections with Urbano.
        """
        return {
            'user': self.user,
            'pass': self.password,
            'Content-Type': 'application/x-www-form-urlencoded',
        }

    def get_shipping_label(self):
        raise NotImplementedError(
            'get_shipping_label is not a method implemented for UrbanoHandler')

    def get_default_payload(self, instance):
        raise NotImplementedError(
            'get_default_payload is not a method implemented for UrbanoHandler')

    def create_shipping(self):
        raise NotImplementedError(
            'create_shipping is not a method implemented for UrbanoHandler')

    def get_tracking(self, identifier):
        """
            This method obtain a detail a shipping of Urbano.
        """
        try:
            data = {
                'guia': '',
                'docref': f'{identifier}',
                'vp_linea': '3'
            }
            params = {'json': json.dumps(data)}
            url = f'{self.base_url}ws/ue/tracking/?{urllib.parse.urlencode(params)}'
            response = self.connector.get(url)[0]
            logger.debug(response)

            if not response['sql_error'] == '1':
                raise ConnectorException(
                    response['msg_error'], 'Error requesting tracking', response['sql_error']
                )

            return response

        except ConnectorException as error:
            logger.error(error)
            raise ConnectorException(error.message, error.description, error.code) from error
